-- EasyColorPicker v2.0
-----------------------

local ColorPickerFrame, OpacitySliderFrame, ColorSwatch = _G["ColorPickerFrame"], _G["OpacitySliderFrame"], _G["ColorSwatch"]

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
	tile = true, edgeSize = 1, tileSize = 5,
}

-- allow draggability
local anchor = CreateFrame("Frame", "ColorPickerFrameAnchor", ColorPickerFrame)
anchor:SetSize(120, 24)
anchor:SetPoint("Top", ColorPickerFrame, "Top", 0, 4)
anchor:EnableMouse(true)
anchor:SetMovable(true)

anchor:SetScript("OnMouseDown", function(self)
		if self:IsMovable() then
			ColorPickerFrame:StartMoving()
		end
	end)

anchor:SetScript("OnMouseUp", function(self)
		if self:IsMovable() then
			ColorPickerFrame:StopMovingOrSizing()
		end
	end)

-- make it lockable

----------------------------------------------------------
-- error template

local errorFrame = CreateFrame("Frame", "ColorPickerFrameErrorFrame", ColorPickerFrame)
errorFrame:SetSize(100, 44)
errorFrame:SetPoint("Bottomright", ColorPickerFrame, "Topright", -10, 0)

errorFrame:SetBackdrop(backdrop)
errorFrame:SetBackdropColor(0, 0, 0, 0.5)
errorFrame:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)

errorFrame.errorString = errorFrame:CreateFontString(nil, "Artwork", "GameFontNormalSmall")
errorFrame.errorString:SetPoint("Topleft", errorFrame, "Topleft", 0, 0)
errorFrame.errorString:SetPoint("Bottomright", errorFrame, "Bottomright", 0, 0)
errorFrame.errorString:SetTextColor(1, 1, 1)

errorFrame:Hide()

function errorFrame:Assign(formatString, errorMessage, isError)
	--if ( isError ) then 

	--end

	self.errorString:SetFormattedText(formatString, errorMessage)
	self:Show()
end

function errorFrame:Clear()
	self:Hide()
end
----------------------------------------------------------

local function roundToDecimal(decimal, integer, returnAsNumber)
	local formatString = "%." .. decimal .. "f"
	local roundedString = string.format(formatString, integer)
	return returnAsNumber and tonumber(roundedValue) or roundedString
end

local function buildHexString(...)
	local hexString = ""
	for i=1,3 do 
		if select(i, ...) < 16 then
			hexString = string.format("%s%s%s", hexString, 0, format("%X", select(i, ...)))
		else
			hexString = string.format("%s%s", hexString, format("%X", select(i, ...)))
		end
	end
	return hexString
end

local function getHexString()
	--local r, g, b = ColorPickerFrameEditBoxRed:GetNumber(), ColorPickerFrameEditBoxGreen:GetNumber(), ColorPickerFrameEditBlue:GetNumber()
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local hexString = buildHexString(r, g, b)

	return hexString
end

-- standard calls for scripts
local function EditBox_OnEnterPressed(frame)
	frame.oldValue = frame:GetText()
	PlaySound("igMainMenuOptionCheckBoxOn")
	frame:HighlightText(0, 0)
	frame:ClearFocus()
end

local function EditBox_OnEscapePressed(frame)
	-- need to revert changes here!
	-- restore old editbox text in case it got deleted
	ColorPickerFrame:SetColorRGB(ColorSwatch2.r, ColorSwatch2.g, ColorSwatch2.b)
	-- OpacitySliderFrame:SetValue

	frame:SetText(frame.oldValue)
	frame:HighlightText(0, 0)
	frame:ClearFocus()
end

local function EditBox_OnEnter(frame)
	frame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
end

local function EditBox_OnLeave(frame)
	frame:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)
end

local function EditBox_OnEditFocusGained(frame)
	-- want to store original value of the editbox
	frame.oldValue = frame:GetText()
	-- in this case we never want to de-highlight the currently focused editbox
	-- even if we're mouseovering another editbox
	frame:SetScript("OnLeave", nil)
	EditBox_OnEnter(frame)
end

local function EditBox_OnEditFocusLost(frame)
	errorFrame:Clear()
	frame:SetText(frame.oldValue)
	frame:SetScript("OnLeave", EditBox_OnLeave)
	EditBox_OnLeave(frame)
end

do
	----------------------------------------------

	--XXX OnEnterPressed necessary, if on the fly adjustment are done on OnChar?!?

	-- red, green, blue editboxes:
	local function RGB_OnEnterPressed(frame)
		EditBox_OnEnterPressed(frame)
		r, g, b = ColorPickerFrameEditBoxRed:GetNumber()/255, ColorPickerFrameEditBoxGreen:GetNumber()/255, ColorPickerFrameEditBoxBlue:GetNumber()/255
		ColorSwatch2.r, ColorSwatch2.g, ColorSwatch.b = r, g, b
		if ( frame:GetNumber() > 255 ) then frame:SetText(255) end
	end

	-- hex editbox:
	--[[
	local function Hex_OnEnterPressed(frame)
		EditBox_OnEnterPressed(frame)

		local value = frame:GetText()
		local r, g, b = tonumber(value:sub(1,2), 16)/255, tonumber(value:sub(3,4), 16)/255, tonumber(value:sub(5,6), 16)/255 

		ColorPickerFrame:SetColorRGB(r, g, b)
	end
	]]

	-- opacity editbox:
	local function Opacity_OnEnterPressed(frame)
		EditBox_OnEnterPressed(frame)
		local alpha = tonumber(ColorPickerFrameEditBoxOpacity:GetText():format("%.3f"))

		--XXX does it matter??
		alpha = alpha < 0 and 0 or alpha > 1 and 1 or alpha
		OpacitySliderFrame:SetValue(1 - alpha)
	end

	----------------------------------------------------------------------

	local r, g, b = ColorPickerFrame:GetColorRGB() -- buildHexString(r*255, g*255, b*255)
	local hex, alpha = getHexString(), OpacitySliderFrame:GetValue()

	local order = {
		[1] = "Red",
		[2] = "Green",
		[3] = "Blue",
		[4] = "Hex",
		[5] = "Opacity",
	}

	local editboxValues = {
	["Red"] 	= { text = "R", value = r*255 , parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 40, point = { "Bottomleft", ColorPickerFrame, "Bottomleft", 28, 38 }, OnEnterPressed = RGB_OnEnterPressed,		},
	["Green"] 	= { text = "G", value = g*255 , parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 40, point = { "Left",		  "Red",			"Left",		  58, 0	 }, OnEnterPressed = RGB_OnEnterPressed,		},
	["Blue"]	= { text = "B", value = b*255 , parent = ColorPickerFrame,   numeric = true,  maxLetters = 3, width = 40, point = { "Left",		  "Green",			"Left",		  58, 0	 }, OnEnterPressed = RGB_OnEnterPressed,		},
	["Hex"]		= { text = "#", value = hex   , parent = ColorPickerFrame,   numeric = false, maxLetters = 6, width = 58, point = { "Left",		  "Blue",			"Left", 	  58, 0	 }, 											},
	["Opacity"]	= { text = "A", value = alpha , parent = OpacitySliderFrame, numeric = false, maxLetters = 5, width = 50, point = { "Left",		  "Hex",			"Left", 	  80, 0	 },	OnEnterPressed = Opacity_OnEnterPressed,	},
	--["Hue"] 	= { text = "H", value = Hue, parent = ColorPickerFrame, numeric = true, maxLetters = 3, width = 40 },
	--["Satu"] 	= { text = "S", value = saturation, parent = ColorPickerFrame, numeric = true, maxLetters = 3, width = 40 },
	--["Value"] = { text = "V", value = hue, parent = ColorPickerFrame, numeric = true, maxLetters = 3, width = 40 },

	}


	editboxValues.Red.OnChar = function(self, text)
		local editbox = "ColorPickerFrameEditBox"
		local r, g, b = _G[editbox.."Red"]:GetNumber()/255, _G[editbox.."Green"]:GetNumber()/255, _G[editbox.."Blue"]:GetNumber()/255 
		--local editboxText = self:GetText()
		self.disableUpdate = true

		if ( self:GetNumber() > 255 ) then
			errorFrame:Assign("You may not use\nnumbers > |cFFFF0000%s|r\n in this editbox!", 255)
		end

		print(string.format("%s: red - %s, green - %s, blue - %s", self:GetName(), r, g, b))
		ColorPickerFrame:SetColorRGB(r, g, b)
	end

	editboxValues.Green.OnChar = editboxValues.Red.OnChar
	editboxValues.Blue.OnChar = editboxValues.Red.OnChar

	editboxValues.Hex.OnChar = function(self, text)
		local editboxText = self:GetText()

		--[^0-9a-fA-F]
		if ( text:find("[^%x]") ) then
			errorFrame:Assign("You may not use\n |cFFFF0000%s|r \nin this editbox!", text:upper(), true)
			-- ignore everything which isn't [0-9], [a-f] or [A-F]
			self:SetText(editboxText:gsub("[^%x]", ""))
		else
			if ( text:find("[a-f]") ) then
				-- make everything uppercase
				self:SetText(self:GetText():upper())
			end

			local hexValue = self:GetText()
			self.disableUpdate = true

			for i = hexValue:len(), 5 do 
				hexValue = hexValue .. 0
			end

			local r, g, b = tonumber(hexValue:sub(1,2), 16)/255, tonumber(hexValue:sub(3,4), 16)/255, tonumber(hexValue:sub(5,6), 16)/255 
			ColorPickerFrame:SetColorRGB(r, g, b)
		end
	
	end

	editboxValues.Opacity.OnChar = function(self, text)
		local editboxText = self:GetText()

		if ( text:find("[^%d.]") ) then
			errorFrame:Assign("You may not use\n |cFFFF0000%s|r \nin this editbox!", text:upper(), true)
			-- ignore anything which isn't [0-9] or [.]
			self:SetText(editboxText:gsub("[^%d.]", ""))
		else
			self.disableUpdate = true
		end

	end

	-------------------------------------------------------------------------------

	local function setupColorPicker(self)
		ColorPickerFrame:SetHeight(220)
		ColorPickerFrame:EnableKeyboard(false)

		local backdrop = {
			bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
			tile = true, edgeSize = 1, tileSize = 5,
		}

		for i=1,#order do 
			local k = order[i] 
			local v = editboxValues[k]

			local editbox = CreateFrame("EditBox", "ColorPickerFrameEditBox"..k, v.parent)

			editbox:SetAutoFocus(false)
			editbox:SetFontObject(GameFontHighlightSmall)
			editbox:SetJustifyH("CENTER")
			editbox:EnableMouse(true)
			editbox:SetBackdrop(backdrop)
			editbox:SetBackdropColor(0, 0, 0, 0.5)
			editbox:SetBackdropBorderColor(0.3, 0.3, 0.3, 0.8)
			editbox:SetText(v.value)

			-- script handler
			editbox:SetScript("OnEnter", EditBox_OnEnter)
			editbox:SetScript("OnLeave", EditBox_OnLeave)
			editbox:SetScript("OnEditFocusGained", EditBox_OnEditFocusGained)
			editbox:SetScript("OnEditFocusLost", EditBox_OnEditFocusLost)
		
			editbox:SetScript("OnEnterPressed", v.OnEnterPressed and v.OnEnterPressed or EditBox_OnEnterPressed)
			editbox:SetScript("OnEscapePressed", EditBox_OnEscapePressed)
					
			if v.OnChar then editbox:SetScript("OnChar", v.OnChar) end

			local relativeTo = type(v.point[2]) == "string" and _G["ColorPickerFrameEditBox"..v.point[2]] or v.point[2]
			editbox:SetPoint(v.point[1], relativeTo, v.point[3], v.point[4], v.point[5])
			editbox:SetNumeric(v.numeric)
			editbox:SetMaxLetters(v.maxLetters)
			editbox:SetSize(v.width, 14)


			local text = editbox:CreateFontString(nil, "Overlay", "GameFontNormal")
			text:SetSize(24, 24)
			text:SetJustifyH("LEFT")
			text:SetTextColor(1, 1, 1)
			text:SetPoint("Right", editbox, "Left", 12, 0)
			text:SetText(v.text)
		end

		-- reanchor ColorSwatchFrame
		ColorSwatch:ClearAllPoints()
		ColorSwatch:SetPoint("Topleft", ColorPickerFrame, "Topleft", 225, -47)
		ColorSwatch.title = ColorPickerFrame:CreateFontString(nil, "Overlay", "GameFontNormalSmall")
		ColorSwatch.title:SetPoint("Bottomleft", "ColorSwatch", "Topleft", -13, 2)
		ColorSwatch.title:SetText("New:")

		-- reanchor OpacitySliderFrame
		OpacitySliderFrame:ClearAllPoints()
		OpacitySliderFrame:SetPoint("Topleft", "ColorSwatch", "Topright", 32, 19)
		OpacitySliderFrame:SetValueStep(0.001)
		OpacitySliderFrame:SetObeyStepOnDrag(true)

		-- ColorSwatch2
		local function ColorSwitch2_OnClick(self, button, down)
			local r, g, b = ColorSwatch2.r, ColorSwatch2.g, ColorSwatch2.b
			ColorPickerFrame:SetColorRGB(r, g, b)
		end

		local colorswatch2 = CreateFrame("Button", "ColorSwatch2", ColorPickerFrame)
		colorswatch2:SetSize(32, 32)
		colorswatch2:SetPoint("Topleft", ColorSwatch, "Bottomleft", 0, -32)
		colorswatch2:SetScript("OnClick", ColorSwitch2_OnClick)
		colorswatch2.title = colorswatch2:CreateFontString(nil, "Overlay", "GameFontNormalSmall")
		colorswatch2.title:SetPoint("Bottomleft", colorswatch2, "Topleft", -13, 2)
		colorswatch2.title:SetText("Old:")

		colorswatch2.info = colorswatch2:CreateFontString(nil, "Overlay", "GameFontNormalSmall")
		colorswatch2.info:SetPoint("Topleft", colorswatch2, "Bottomleft", -5, -2)

		colorswatch2.texture = colorswatch2:CreateTexture("ColorSwatch2", "Artwork")
		colorswatch2.texture:SetAllPoints(colorswatch2)


		setupColorPicker = nil
	end
	setupColorPicker()
end

-- hook OnColorSelect
ColorPickerFrame:HookScript("OnColorSelect", function(self, r, g, b)
	local ColorPickerFrameEditBoxRed = ColorPickerFrameEditBoxRed
	local ColorPickerFrameEditBoxGreen = ColorPickerFrameEditBoxGreen
	local ColorPickerFrameEditBoxBlue = ColorPickerFrameEditBoxBlue
	local ColorPickerFrameEditBoxHex = ColorPickerFrameEditBoxHex

	local hex = buildHexString(r*255, g*255, b*255)

	if not ColorPickerFrameEditBoxRed.disableUpdate then ColorPickerFrameEditBoxRed:SetText(r*255) end
	if not ColorPickerFrameEditBoxGreen.disableUpdate then ColorPickerFrameEditBoxGreen:SetText(g*255) end
	if not ColorPickerFrameEditBoxBlue.disableUpdate then ColorPickerFrameEditBoxBlue:SetText(b*255) end
	if not ColorPickerFrameEditBoxHex.disableUpdate then ColorPickerFrameEditBoxHex:SetText(hex) end

	ColorPickerFrameEditBoxRed.disableUpdate = false
	ColorPickerFrameEditBoxGreen.disableUpdate = false
	ColorPickerFrameEditBoxBlue.disableUpdate = false
	ColorPickerFrameEditBoxHex.disableUpdate = false
	
end)

ColorPickerFrame:HookScript("OnShow", function(self)
	local colorswatch2 = _G["ColorSwatch2"]
	local r, g, b = ColorPickerFrame:GetColorRGB()
	local hex = buildHexString(r*255, g*255, b*255)

	-- current set values:
	--ColorSwatch.r, ColorSwatch.g, ColorSwatch.b = r, g, b
	colorswatch2.r, colorswatch2.g, colorswatch2.b = r, g, b
	colorswatch2.texture:SetTexture(r, g, b)
	colorswatch2.info:SetText(hex)
end)

OpacitySliderFrame:HookScript("OnValueChanged", function(self, value)
	local value = 1 - value
	ColorPickerFrameEditBoxOpacity:SetText(roundToDecimal(3, value))
end)